package game

import (
	"fmt"
	"sirlab.de/go/knyt/syncdata"
	"sort"
)

func (gm *Game) populateSyncDataCb(syncData *syncdata.SyncData) {
	syncData.GameInfo = gm.populateGameInfo()
}

func (gm *Game) populateGameInfo() syncdata.GameInfo {
	gm.mu.Lock()
	defer gm.mu.Unlock()

	return syncdata.GameInfo{
		GameId:  gm.id,
		State:   gm.state,
		Phase:   gm.phase,
		Players: gm.getPlayers(),
		Round:   gm.getRoundInfo(),
	}
}

func (gm *Game) getPlayers() []syncdata.PlayerInfo {
	players := make([]syncdata.PlayerInfo, 0)
	for _, p := range gm.players {
		if p.isPlaying {
			players = append(players, syncdata.PlayerInfo{
				Id:     p.id,
				Name:   p.name,
				IsIdle: p.isIdle,
			})
		}
	}

	sort.Slice(players, func(a, b int) bool {
		return players[a].Name < players[b].Name
	})

	return players
}

func (gm *Game) getRoundInfo() *syncdata.RoundInfo {
	if gm.state != STATE_PLAY {
		return nil
	}

	quote := gm.quotes[gm.round.quoteId]
	if quote == nil {
		fmt.Printf("fatal error: quote id \"%s\" not found\n", gm.round.quoteId)
		return nil
	}

	sources := make([]syncdata.SourceInfo, 0)
	for _, source := range gm.round.sources {
		sources = append(sources, syncdata.SourceInfo{
			Id:   source.id,
			Name: source.name,
		})
	}

	playerState := make([]syncdata.PlayerState, 0)
	for id, playerInfo := range gm.players {
		if !playerInfo.isPlaying {
			continue
		}

		state := syncdata.PLAYER_STATE_UNDECIDED
		selection := gm.round.selection[id]
		if len(selection) > 0 {
			state = syncdata.PLAYER_STATE_DECIDED
		} else if playerInfo.isIdle {
			state = syncdata.PLAYER_STATE_IDLE
		}

		playerState = append(playerState, syncdata.PlayerState{
			Id:    id,
			State: state,
		})
	}

	roundInfo := syncdata.RoundInfo{
		Quote:       quote.GetQuote(),
		Sources:     sources,
		PlayerState: playerState,
	}

	return &roundInfo
}
